export = eventNames;
/**
 * The name of all the events {@link Jimpex} can trigger.
 *
 * @typedef {Object} JimpexEvents
 * @property {string} beforeStop               Called before closing the instance of the
 *                                             app.
 * @property {string} start                    Called from the `listen` callback, when the
 *                                             app is ready to be used.
 * @property {string} afterStart               Called from the `listen` callback, when all
 *                                             controllers and middlewares have been
 *                                             mounted.
 * @property {string} afterStartCallback       Called right after the callback sent to
 *                                             `start`
 *                                             gets executed.
 * @property {string} beforeStart              Called before `listen` is executed on the
 *                                             Express app.
 * @property {string} afterStop                Called after the app instance has been
 *                                             closed and deleted.
 * @property {string} routeAdded               Called every time a new route is added to
 *                                             the app.
 * @property {string} controllerWillBeMounted  This is for a reducer event. It gets called
 *                                             before mounting a router or a set of routes
 *                                             to the app in order to "reduce it".
 * @property {string} middlewareWillBeUsed     This is for a reducer event. It gets called
 *                                             before using a middleware in order to
 *                                             "reduce it".
 * @parent module:constants
 */
/**
 * The name of all the events {@link Jimpex} can trigger.
 *
 * @type {JimpexEvents}
 * @parent module:constants
 */
declare const eventNames: JimpexEvents;
declare namespace eventNames {
    export { JimpexEvents };
}
/**
 * The name of all the events {@link Jimpex} can trigger.
 */
type JimpexEvents = {
    /**
     * Called before closing the instance of the
     * app.
     */
    beforeStop: string;
    /**
     * Called from the `listen` callback, when the
     * app is ready to be used.
     */
    start: string;
    /**
     * Called from the `listen` callback, when all
     * controllers and middlewares have been
     * mounted.
     */
    afterStart: string;
    /**
     * Called right after the callback sent to
     * `start`
     * gets executed.
     */
    afterStartCallback: string;
    /**
     * Called before `listen` is executed on the
     * Express app.
     */
    beforeStart: string;
    /**
     * Called after the app instance has been
     * closed and deleted.
     */
    afterStop: string;
    /**
     * Called every time a new route is added to
     * the app.
     */
    routeAdded: string;
    /**
     * This is for a reducer event. It gets called
     * before mounting a router or a set of routes
     * to the app in order to "reduce it".
     */
    controllerWillBeMounted: string;
    /**
     * This is for a reducer event. It gets called
     * before using a middleware in order to
     * "reduce it".
     */
    middlewareWillBeUsed: string;
};
