export type ExpressMiddleware = import("express").RequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>> | import("express").ErrorRequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>>;
export type ExpressResponse = import("express").Response<any, Record<string, any>>;
export type AppConfiguration = import("wootils/esm/node/appConfiguration").AppConfiguration;
export type ResponsesBuilder = import("../../services/http/responsesBuilder").ResponsesBuilder;
export type Controller = {
    /**
     * The function Jimpex calls when mounting the
     * controller.
     */
    connect: import("../../types").ControllerConnectFn;
    /**
     * A flag set to `true` to identify the
     * resource as a routes controller.
     */
    controller: boolean;
};
/**
 * @typedef {import('../../types').ExpressMiddleware} ExpressMiddleware
 * @typedef {import('../../types').ExpressResponse} ExpressResponse
 * @typedef {import('../../types').AppConfiguration} AppConfiguration
 * @typedef {import('../../services/http/responsesBuilder').ResponsesBuilder} ResponsesBuilder
 * @typedef {import('../../types').Controller} Controller
 * @prettierignore
 */
/**
 * Provides the handlers and middlwares to show and switch configurations.
 *
 * @parent module:controllers
 */
export class ConfigurationController {
    /**
     * @param {AppConfiguration} appConfiguration  To read the app configuration.
     * @param {ResponsesBuilder} responsesBuilder  To generate the JSON responses.
     */
    constructor(appConfiguration: AppConfiguration, responsesBuilder: ResponsesBuilder);
    /**
     * A local reference for the `appConfiguration` service.
     *
     * @type {AppConfiguration}
     * @access protected
     * @ignore
     */
    _appConfiguration: AppConfiguration;
    /**
     * A local reference for the `responsesBuilder` service.
     *
     * @type {ResponsesBuilder}
     * @access protected
     * @ignore
     */
    _responsesBuilder: ResponsesBuilder;
    /**
     * Send a response with the current app configuration as a body.
     *
     * @param {ExpressResponse} res  The server response.
     */
    getConfigurationResponse(res: ExpressResponse): void;
    /**
     * Returns the middleware to show the current configuration.
     *
     * @returns {ExpressMiddleware}
     */
    showConfiguration(): ExpressMiddleware;
    /**
     * Returns the middleware to switch the current configuration.
     *
     * @returns {ExpressMiddleware}
     */
    switchConfiguration(): ExpressMiddleware;
}
/**
 * This controller is kind of special as it will only mount the routes if the
 * `debug.configurationController` setting of the app configuration is `true`.
 * It provides routes for:
 * - Showing the current configuration.
 * - Switching the configuration, but only if the service allows it.
 *
 * @type {Controller}
 * @parent module:controllers
 * @todo Don't use an array to return the routes.
 */
export const configurationController: Controller;
