export type ExpressMiddleware = import("express").RequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>> | import("express").ErrorRequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>>;
export type AppConfiguration = import("wootils/esm/node/appConfiguration").AppConfiguration;
export type Controller = {
    /**
     * The function Jimpex calls when mounting the
     * controller.
     */
    connect: import("../../types").ControllerConnectFn;
    /**
     * A flag set to `true` to identify the
     * resource as a routes controller.
     */
    controller: boolean;
};
export type ResponsesBuilder = import("../../services/http/responsesBuilder").ResponsesBuilder;
/**
 * @typedef {import('../../types').ExpressMiddleware} ExpressMiddleware
 * @typedef {import('../../types').AppConfiguration} AppConfiguration
 * @typedef {import('../../types').Controller} Controller
 * @typedef {import('../../services/http/responsesBuilder').ResponsesBuilder} ResponsesBuilder
 * @prettierignore
 */
/**
 * Provides the handler to show a some minimal health information about the app:
 * - app version.
 * - configuration name.
 *
 * @parent module:controllers
 */
export class HealthController {
    /**
     * @param {AppConfiguration} appConfiguration  To read the app version and the
     *                                             configuration name.
     * @param {ResponsesBuilder} responsesBuilder  To generate the JSON response.
     */
    constructor(appConfiguration: AppConfiguration, responsesBuilder: ResponsesBuilder);
    /**
     * A local reference for the `appConfiguration` service.
     *
     * @type {AppConfiguration}
     * @access protected
     * @ignore
     */
    _appConfiguration: AppConfiguration;
    /**
     * A local reference for the `responsesBuilder` service.
     *
     * @type {ResponsesBuilder}
     * @access protected
     * @ignore
     */
    _responsesBuilder: ResponsesBuilder;
    /**
     * Returns the middleware that shows the health information.
     *
     * @returns {ExpressMiddleware}
     */
    health(): ExpressMiddleware;
}
/**
 * Mounts the health route.
 *
 * @type {Controller}
 * @parent module:controllers
 */
export const healthController: Controller;
