export type Jimpex = import("../../app").Jimpex;
export type ExpressMiddleware = import("express").RequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>> | import("express").ErrorRequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>>;
export type ExpressRouter = import("express").Router;
export type Controller = {
    /**
     * The function Jimpex calls when mounting the
     * controller.
     */
    connect: import("../../types").ControllerConnectFn;
    /**
     * A flag set to `true` to identify the
     * resource as a routes controller.
     */
    controller: boolean;
};
export type MiddlewareLike = import("../../types").Middleware | import("express").RequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>> | import("express").ErrorRequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>>;
export type SendFile = (res: import("express").Response<any, Record<string, any>>, filepath: string, next: import("express").NextFunction, from?: string) => any;
/**
 * <O>
 */
export type ControllerCreator<O> = (options: Partial<O>) => import("../../types").Controller | import("../../types").ControllerProvider;
/**
 * If you wan to customize how, to, and from where files are served, instead of just
 * sending a list of strings, you can use an object with these properties.
 */
export type StaticsControllerFile = {
    /**
     * The route the controller will use for the
     * file.
     */
    route: string;
    /**
     * The path for the file, relative to the
     * root of the app.
     */
    path: string;
    /**
     * A dictionary of custom headers to send on
     * the file response.
     */
    headers: {
        [x: string]: string;
    };
};
/**
 * They are like "master paths" that get prepended to all the file paths and routes the
 * controller use.
 */
export type StaticsControllerPathsOptions = {
    /**
     * A custom route to prefix all the file routes.
     */
    route: string;
    /**
     * A custom path to prefix all the file paths.
     */
    source: string;
};
export type StaticsControllerFileLike = string | StaticsControllerFile;
/**
 * These are the options that allow you to customize the controller, how, to and from
 * where the files are served.
 */
export type StaticsControllerOptions = {
    /**
     * A list of filenames or
     * {@link StaticsControllerFile}
     * definitions.
     */
    files: StaticsControllerFileLike[];
    /**
     * A dictionary of all the HTTP methods
     * the controller will use in order to
     * serve the files. If `all` is set to
     * true, all the other flags will be
     * ignored.
     */
    methods: {
        [x: string]: boolean;
    };
    /**
     * The "master paths" the controller
     * uses to prefix all file routes and
     * paths.
     */
    paths: StaticsControllerPathsOptions;
};
export type StaticsControllerWrapperOptions = StaticsControllerOptions & StaticsControllerWrapperOptionsProperties;
export type StaticsControllerWrapperOptionsProperties = {
    /**
     * Function can be used to add custom middlewares on the file routes. If implemented, it
     * must return a list of middlewares when executed.
     */
    middlewares: StaticsControllerMiddlewaresFn;
};
export type StaticsControllerMiddlewaresFn = (app: Jimpex) => MiddlewareLike[];
/**
 * @typedef {import('../../app').Jimpex} Jimpex
 * @typedef {import('../../types').ExpressMiddleware} ExpressMiddleware
 * @typedef {import('../../types').ExpressRouter} ExpressRouter
 * @typedef {import('../../types').Controller} Controller
 * @typedef {import('../../types').MiddlewareLike} MiddlewareLike
 * @typedef {import('../../services/common/sendFile').SendFile} SendFile
 */
/**
 * @typedef {import('../../types').ControllerCreator<O>} ControllerCreator<O>
 * @template O
 */
/**
 * If you wan to customize how, to, and from where files are served, instead of just
 * sending a list of strings, you can use an object with these properties.
 *
 * @typedef {Object} StaticsControllerFile
 * @property {string}                  route    The route the controller will use for the
 *                                              file.
 * @property {string}                  path     The path for the file, relative to the
 *                                              root of the app.
 * @property {Object.<string, string>} headers  A dictionary of custom headers to send on
 *                                              the file response.
 * @parent module:controllers
 */
/**
 * They are like "master paths" that get prepended to all the file paths and routes the
 * controller use.
 *
 * @typedef {Object} StaticsControllerPathsOptions
 * @property {string} route   A custom route to prefix all the file routes.
 * @property {string} source  A custom path to prefix all the file paths.
 * @parent module:controllers
 */
/**
 * @typedef {string | StaticsControllerFile} StaticsControllerFileLike
 * @parent module:controllers
 */
/**
 * These are the options that allow you to customize the controller, how, to and from
 * where the files are served.
 *
 * @typedef {Object} StaticsControllerOptions
 * @property {StaticsControllerFileLike[]}   files    A list of filenames or
 *                                                    {@link StaticsControllerFile}
 *                                                    definitions.
 * @property {Object.<string, boolean>}      methods  A dictionary of all the HTTP methods
 *                                                    the controller will use in order to
 *                                                    serve the files. If `all` is set to
 *                                                    true, all the other flags will be
 *                                                    ignored.
 * @property {StaticsControllerPathsOptions} paths    The "master paths" the controller
 *                                                    uses to prefix all file routes and
 *                                                    paths.
 * @parent module:controllers
 */
/**
 * @typedef {StaticsControllerOptions & StaticsControllerWrapperOptionsProperties} StaticsControllerWrapperOptions
 * @parent module:controllers
 * @prettierignore
 */
/**
 * @typedef {Object} StaticsControllerWrapperOptionsProperties
 * @property {StaticsControllerMiddlewaresFn} middlewares
 * Function can be used to add custom middlewares on the file routes. If implemented, it
 * must return a list of middlewares when executed.
 * @parent module:controllers
 */
/**
 * @callback StaticsControllerMiddlewaresFn
 * @param {Jimpex} app  A reference for the container.
 * @returns {MiddlewareLike[]}
 * @parent module:controllers
 */
/**
 * This controller allows you to serve specific files from any folder to any route without
 * the need of mounting directories as "static".
 *
 * @parent module:controllers
 */
export class StaticsController {
    /**
     * @param {SendFile} sendFile
     * To send the responses for the files.
     * @param {Partial<StaticsControllerOptions>} options
     * The options to customize the controller.
     */
    constructor(sendFile: SendFile, options?: Partial<StaticsControllerOptions>);
    /**
     * A local reference for the `sendFile` service.
     *
     * @type {SendFile}
     * @access protected
     * @ignore
     */
    _sendFile: SendFile;
    /**
     * The controller configuration options.
     *
     * @type {StaticsControllerOptions}
     * @access protected
     * @ignore
     */
    _options: StaticsControllerOptions;
    /**
     * A dictionary of all the formatted files ({@link StaticsControllerFile}). It uses
     * the files routes as keys.
     *
     * @type {Object.<string, StaticsControllerFile>}
     * @access protected
     * @ignore
     */
    _files: {
        [x: string]: StaticsControllerFile;
    };
    /**
     * Defines all the needed routes to serve the files.
     *
     * @param {ExpressRouter}       router            To generate the routes.
     * @param {ExpressMiddleware[]} [middlewares=[]]  A list of custom middlewares that will
     *                                                be added before the one that serves a
     *                                                file.
     * @returns {ExpressRouter}
     */
    addRoutes(router: ExpressRouter, middlewares?: ExpressMiddleware[]): ExpressRouter;
    /**
     * The controller configuration options.
     *
     * @type {StaticsControllerOptions}
     * @todo Remove Object.freeze.
     */
    get options(): StaticsControllerOptions;
    /**
     * Generates a route for an specific file.
     *
     * @param {ExpressRouter}         router          To create the actual route.
     * @param {string}                method          The HTTP method for the route.
     * @param {StaticsControllerFile} file            The file information.
     * @param {ExpressMiddleware}     fileMiddleware  The middleware that serves the file.
     * @param {ExpressMiddleware[]}   middlewares     A list of custom middlewares to add
     *                                                before the one that serves the file.
     * @returns {ExpressRouter}
     * @access protected
     * @ignore
     */
    _addRoute(router: ExpressRouter, method: string, file: StaticsControllerFile, fileMiddleware: ExpressMiddleware, middlewares: ExpressMiddleware[]): ExpressRouter;
    /**
     * Parses each of the received files in order to create a {@link StaticsControllerFile}.
     *
     * @returns {Object.<string, StaticsControllerFile>}
     * @access protected
     * @ignore
     */
    _createFiles(): {
        [x: string]: StaticsControllerFile;
    };
    /**
     * Generates the middleware to serve a specific file.
     *
     * @param {StaticsControllerFile} file  The file information.
     * @returns {ExpressMiddleware}
     * @access protected
     * @ignore
     */
    _getMiddleware(file: StaticsControllerFile): ExpressMiddleware;
    /**
     * Helper method that validates and normalizes the options received by the controller.
     *
     * @param {StaticsControllerOptions} options  The options to validate.
     * @returns {StaticsControllerOptions}
     * @throws {Error} If no files are specified.
     * @throws {Error} If methods is not defined.
     * @throws {Error} If no methods are enabled.
     * @throws {Error} If there's an invalid HTTP method.
     * @access protected
     * @ignore
     */
    _normalizeOptions(options: StaticsControllerOptions): StaticsControllerOptions;
}
/**
 * This controller allows you to serve specific files from any folder to any route without
 * the need of mounting directories as "static".
 *
 * @type {ControllerCreator<StaticsControllerWrapperOptions>}
 * @parent module:controllers
 */
export const staticsController: ControllerCreator<StaticsControllerWrapperOptions>;
