export type ExpressMiddleware = import("express").RequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>> | import("express").ErrorRequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, qs.ParsedQs, Record<string, any>>;
export type ExpressResponse = import("express").Response<any, Record<string, any>>;
export type ExpressNext = import("express").NextFunction;
export type SendFile = (res: import("express").Response<any, Record<string, any>>, filepath: string, next: import("express").NextFunction, from?: string) => any;
export type HTMLGenerator = import("../../services/html/htmlGenerator").HTMLGenerator;
/**
 * <O>
 */
export type MiddlewareCreator<O> = (options: Partial<O>) => import("../../types").Middleware;
/**
 * A set options to customize the middleware behavior.
 */
export type ShowHTMLMiddlewareOptions = {
    /**
     * The name of the file the middleware will serve.
     * Default `'index.html'`.
     */
    file: string;
    /**
     * The name of a {@link HTMLGenerator} service for the
     * middleware to use. If the service is available, the
     * value of `file` will be overwritten for the file
     * generated by the service. Default `'htmlGenerator'`.
     */
    htmlGenerator: string;
};
/**
 * @typedef {import('../../types').ExpressMiddleware} ExpressMiddleware
 * @typedef {import('../../types').ExpressResponse} ExpressResponse
 * @typedef {import('../../types').ExpressNext} ExpressNext
 * @typedef {import('../../services/common/sendFile').SendFile} SendFile
 * @typedef {import('../../services/html/htmlGenerator').HTMLGenerator} HTMLGenerator
 */
/**
 * @typedef {import('../../types').MiddlewareCreator<O>} MiddlewareCreator<O>
 * @template O
 */
/**
 * A set options to customize the middleware behavior.
 *
 * @typedef {Object} ShowHTMLMiddlewareOptions
 * @property {string} file           The name of the file the middleware will serve.
 *                                   Default `'index.html'`.
 * @property {string} htmlGenerator  The name of a {@link HTMLGenerator} service for the
 *                                   middleware to use. If the service is available, the
 *                                   value of `file` will be overwritten for the file
 *                                   generated by the service. Default `'htmlGenerator'`.
 * @parent module:middlewares
 */
/**
 * A very simple middleware service to send an HTML on a server response. The special
 * _'feature'_ of this service is that it can be hooked up to an `HTMLGenerator` service
 * and it will automatically server the file generated by it.
 *
 * @parent module:middlewares
 */
export class ShowHTML {
    /**
     * @param {SendFile}      sendFile              Necessary to server the HTML file.
     * @param {string}        [file='index.html']   The name of the file it will serve. If
     *                                              `htmlGenerator` is specified, this will
     *                                              be overwritten with the name of the file
     *                                              generated by that service.
     * @param {HTMLGenerator} [htmlGenerator=null]  If used, the file to server will be the
     *                                              one generated by that service.
     */
    constructor(sendFile: SendFile, file?: string, htmlGenerator?: HTMLGenerator);
    /**
     * A local reference for the `sendFile` service.
     *
     * @type {SendFile}
     * @access protected
     * @ignore
     */
    _sendFile: SendFile;
    /**
     * The name of the file to serve.
     *
     * @type {string}
     * @access protected
     * @ignore
     */
    _file: string;
    /**
     * If specified, a reference for a service that generates HTML files.
     *
     * @type {HTMLGenerator}
     * @access protected
     * @ignore
     */
    _htmlGenerator: HTMLGenerator;
    /**
     * Whether or not the file is ready to be served.
     *
     * @type {boolean}
     * @access protected
     * @ignore
     */
    _ready: boolean;
    /**
     * Returns the Express middleware that serves the HTML file.
     *
     * @returns {ExpressMiddleware}
     */
    middleware(): ExpressMiddleware;
    /**
     * The name of the file to serve.
     *
     * @type {string}
     */
    get file(): string;
    /**
     * Serves the file on the response.
     *
     * @param {ExpressResponse} res   The server response.
     * @param {ExpressNext}     next  The functino to call the next middleware.
     * @access protected
     * @ignore
     */
    _sendHTML(res: ExpressResponse, next: ExpressNext): void;
}
/**
 * A middleware for showing an `index.html` file.
 *
 * @type {MiddlewareCreator<ShowHTMLMiddlewareOptions>}
 * @parent module:middlewares
 */
export const showHTML: MiddlewareCreator<ShowHTMLMiddlewareOptions>;
