export type Provider = {
    register: import("wootils/esm/shared/jimpleFns").ProviderRegisterFn;
};
/**
 * @typedef {import('../../types').Provider} Provider
 */
/**
 * A type of error to be used on HTTP requests.
 *
 * @augments AppError
 * @parent module:services
 */
export class HTTPError extends AppError {
    /**
     * @param {string} message       The error message.
     * @param {number} [status=200]  The HTTP status code of the request response.
     * @param {Object} [context={}]  Context information related to the error.
     */
    constructor(message: string, status?: number, context?: any);
}
/**
 * A service provider that will register both the {@link HTTPError} and a generator
 * function on the container. `HTTPError` will be the key for class, and `httpError` will
 * be for the generator function.
 *
 * @type {Provider}
 * @example
 *
 *   // Register it on the container
 *   container.register(httpError);
 *   // Getting access to the class.
 *   const HTTPError = container.get('HTTPError');
 *   // Getting access to the function.
 *   const httpError = container.get('httpError');
 *
 * @parent module:services
 */
export const httpError: Provider;
import { AppError } from "./appError";
